//-----------------------------------------------------------------------------
//
//  Simulator Bob, Copyright (C) 2003-2005 Patrik Stellmann 
//  All rights reserved.
//  
//  email: <stellmann@users.sourceforge.net> web: <simbob.sourceforge.net>
//
//  This program/file is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//  
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details. (http://www.gnu.org)
//
//-----------------------------------------------------------------------------
//
//  $Author: $
//
//  $Date: $
//
//  $Revision: $
//
//  $Source: $
//
//-----------------------------------------------------------------------------
//
//  $Log: $
//-----------------------------------------------------------------------------
#include "stdafx.hpp"
#include "testextension\testextension.hpp"
using namespace TESTEXTENSION;

/*!
	\class TESTEXTENSION::RCFollowWallController
	\author <a href="mailto:stellmann@users.sourceforge.net">Patrik Stellmann</a>
	\brief ...

	...
*/

/*!
*/
TESTEXTENSION::RCFollowWallController::RCFollowWallController(
		const RCString&				sKey,
		const RSControllerCtorArg&	CtorArg)	:
	TRCControllerComponent<RCFollowWallController>(
			*this,
			sKey,
			CtorArg),
	m_LeftSensorPtr		(m_RtlComponentHandler, m_pGlobalList, this),
	m_RightSensorPtr	(m_RtlComponentHandler, m_pGlobalList, this),
	m_MotionControlPtr	(m_RtlComponentHandler, m_pGlobalList, this),
	m_nLinVelocity(0.2),
	m_nRotVelocity(0.5),
	m_nPeriod(0.1),
	m_ThreadPtr(CreateThread())
{
	//	<empty>
}

/*!
*/
TESTEXTENSION::RCFollowWallController::~RCFollowWallController()
{
	m_ThreadPtr->WaitForTermination();
}

/*!
*/
void TESTEXTENSION::RCFollowWallController::Realize()
{
	TRCControllerComponent<RCFollowWallController>::Realize();

	TBool bSuccess = true;
	const RCString sOwner = 
			RCString(_T("follow wall controller'")) + GetFullComponentKey() + _T("'");

	bSuccess &=	m_LeftSensorPtr		.Realize(_T("left sensor"),		sOwner);
	bSuccess &=	m_RightSensorPtr	.Realize(_T("right sensor"),	sOwner);
	bSuccess &=	m_MotionControlPtr	.Realize(_T("motion control"),	sOwner);

	if (!bSuccess)
	{
		throw RCException(
			RCString(_T("Realization of ")) + sOwner + _T(" failed!"));
	}

	m_ThreadPtr->Start(TThreadCallback(*this, ThreadFcn));
}

/*!
*/
void TESTEXTENSION::RCFollowWallController::Clear()
{
	m_LeftSensorPtr.Clear();
	m_RightSensorPtr.Clear();
	m_MotionControlPtr.Clear();

	TRCControllerComponent<RCFollowWallController>::Clear();
}

/*!
*/
TLong TESTEXTENSION::RCFollowWallController::ThreadFcn(void*)
{
	while (!QuitEvent().IsSet())
	{
		if (m_LeftSensorPtr->CheckContact())
		{
			m_MotionControlPtr->SetDestAngVelocity(m_nRotVelocity);
			m_MotionControlPtr->SetDestLinVelocity(0);
		}
		else if (m_RightSensorPtr->CheckContact())
		{
			m_MotionControlPtr->SetDestAngVelocity(-m_nRotVelocity);
			m_MotionControlPtr->SetDestLinVelocity(0);
		}
		else
		{
			m_MotionControlPtr->SetDestAngVelocity(0);
			m_MotionControlPtr->SetDestLinVelocity(m_nLinVelocity);
		}

		Sleep(Round(m_nPeriod * MILLI_INV));
	}
	return 0;
}
